<xsl:stylesheet version="1.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:func="http://exslt.org/functions"
  xmlns:dp="http://www.datapower.com/extensions"
  extension-element-prefixes="dp"
>
  <xsl:output method="text" />

  <xsl:variable name="uri" select="substring(dp:variable('var://service/URI'),2)"/>

  <xsl:variable name="url0" select="concat('http://',substring-after($uri,':'))"/>
  <xsl:variable name="sla"  select="dp:last-index-of($url0, '/')"/>
  <xsl:variable name="url"  select="substring($url0, 1, $sla)"/>
  <xsl:variable name="ini"  select="concat(substring($url0, $sla+1), '?wsdl')"/>
  <xsl:variable name="size" select="substring-before($uri,':')"/>
  <xsl:variable name="nsWsdl" select=" 'http://schemas.xmlsoap.org/wsdl/' "/>
  <xsl:variable name="nsXsd" select=" 'http://www.w3.org/2001/XMLSchema' "/>

  <xsl:template match="/">

<xsl:text><![CDATA[<html><body><script type="text/vnd.graphviz" id="digraph">]]>
digraph G {
rankdir=LR size="</xsl:text><xsl:value-of select="$size"/><xsl:text>" concentrate=true
</xsl:text>

    <xsl:call-template name="scanWsdl">
      <xsl:with-param name="suf" select="$ini"/>
    </xsl:call-template>

<xsl:text>}
<![CDATA[</script>
<script src="data:text/javascript;base64,]]></xsl:text>
    <xsl:variable name="viz.js">
      <dp:url-open 
        target="local:///viz.js"
        response="binaryNode"
      />      
    </xsl:variable>

    <xsl:value-of select="dp:binary-encode($viz.js/result/binary/node())"/>
<xsl:text><![CDATA["></script>
<script>document.body.innerHTML = Viz(document.getElementById("digraph").innerHTML, "svg");</script>
</body></html>]]>
</xsl:text>

    <dp:set-http-request-header name="'Content-Type'" value=" 'text/html' " />
    <dp:freeze-headers/>
    <dp:set-variable name="'var://service/mpgw/skip-backside'" value="'1'"/>
  </xsl:template>


  <xsl:template name="scanWsdl">
    <xsl:param name="suf"/>

    <xsl:variable name="wsdl">
      <xsl:copy-of select="document(concat($url,$suf))"/>
    </xsl:variable>

    <xsl:value-of select="func:node($suf, 'hexagon', $wsdl)"/>

    <xsl:for-each select="$wsdl//*[local-name()='import' and namespace-uri()=$nsWsdl]">
      <xsl:call-template name="scanWsdl">
        <xsl:with-param name="suf" select=" @location "/>
      </xsl:call-template>

      <xsl:value-of select="func:edge($suf, @location)"/>
    </xsl:for-each>

    <xsl:for-each select="$wsdl//*[local-name()='import' and namespace-uri()=$nsXsd] ">
      <xsl:call-template name="scanXsd">
        <xsl:with-param name="suf" select=" @schemaLocation "/>
      </xsl:call-template>

      <xsl:value-of select="func:edge($suf, @schemaLocation)"/>
    </xsl:for-each>
  </xsl:template>

  
  <xsl:template name="scanXsd">
    <xsl:param name="suf"/>

    <xsl:variable name="xsd">
      <xsl:copy-of select="document(concat($url,$suf))"/>
    </xsl:variable>

    <xsl:value-of select="func:node($suf, 'box', $xsd)"/>

    <xsl:for-each select="$xsd//*[local-name()='include' and namespace-uri()=$nsXsd] ">
      <xsl:call-template name="scanXsd">
        <xsl:with-param name="suf" select=" @schemaLocation "/>
      </xsl:call-template>

      <xsl:copy-of select="func:edge($suf, @schemaLocation, true())"/>
    </xsl:for-each>

    <xsl:for-each select="$xsd//*[local-name()='import' and namespace-uri()=$nsXsd] ">
      <xsl:call-template name="scanXsd">
        <xsl:with-param name="suf" select=" @schemaLocation "/>
      </xsl:call-template>

      <xsl:copy-of select="func:edge($suf, @schemaLocation)"/>
    </xsl:for-each>
  </xsl:template>

    
  <func:function name="func:edge">
    <xsl:param name="s"/>
    <xsl:param name="t"/>
    <xsl:param name="d" select="false()"/>

    <func:result>"<xsl:value-of select="$s"/>"->"<xsl:value-of select="$t"/>"<xsl:if test="$d"> [style=dashed]</xsl:if>&#10;</func:result>
  </func:function>

  <func:function name="func:node">
    <xsl:param name="name"/>
    <xsl:param name="shape"/>
    <xsl:param name="doc"/>

    <func:result>"<xsl:value-of select="$name"/>" [shape=<xsl:value-of select="$shape"/> color=<xsl:value-of select="func:color($doc)"/>]&#10;</func:result>
  </func:function>

  <func:function name="func:color">
    <xsl:param name="doc"/>

    <func:result>
      <xsl:choose>
        <xsl:when test="count($doc/*)">blue</xsl:when>
        <xsl:otherwise>red</xsl:otherwise>
      </xsl:choose> 
    </func:result>
  </func:function>

</xsl:stylesheet>
